#include <stdio.h>
#include "sherlock.h"
#include "error.h"

/*This table is tied in with the #defines in error.h.  Don't change one without
  changing the other.
*/
char *errlist[] = {
                    "usage: %s %s\n",
                    "Missing '-i' option",
		    "Can't display image on less than 8 planes",
                    "Filename wasn't given",
		    "Image has unknown format",
		    "Pattern has to be the same size or smaller than the image",
		    "Have to select or load a pattern first"
                  };

/*Display an error message.  Exit if the error is fatal.

  Inputs:  errcode      - error code
	   type         - type of error (warning or fatal)
  Outputs: none
  Locals:  none
  Globals: errlist      - list of error messages
	   errno        - number of last error encountered
	   program      - name of currently executing program
	   stderr       - standard error
	   EBADFORMAT   - "image in unknown format" error
	   ENODEPTH     - "display has less than 8 planes" error
	   ENOIMAGE     - "missing -i option" error
	   ENOPATTERN   - "no pattern in memory" error
	   ERROR_START  - start of program specific errors
	   ETOOBIG      - "pattern too big for given image" error
	   EUSAGE       - "error in usage" error
	   FATAL        - error is fatal, program will terminate
	   SHOW_VERSION - show version number and exit
	   USAGE        - usage message
	   VERSION      - version number
*/
error(type,errcode)
register int type,errcode;
{
	switch(errcode) {
	  case SHOW_VERSION:
		fprintf(stderr,"%s: version %s\n",program,VERSION);
		break;
	  case EUSAGE:
		fprintf(stderr,errlist[errcode - ERROR_START],program,USAGE);
		break;
	  case EBADFORMAT:
	  case ENOIMAGE:
	  case ENODEPTH:
	  case ENOPATTERN:
	  case ETOOBIG:
		fprintf(stderr,"%s: %s\n",program,
			errlist[errcode - ERROR_START]);
		break;
	  default:
		perror(program);
		break;
	}

	/*Cleanup and terminate if the error is fatal*/
	if(type == FATAL) {
	  errno = errcode;
	  cleanup();
	} else errno = 0;
}

