#include "image.h"

/*General constants*/
#define DEF_CORR_RESULTS_FILE "corr_results.data"
#define DEF_DIST_RESULTS_FILE "dist_results.data"
#define USAGE "-i filename [-b {batchfile | -}] [-ii { y | n }] [-ip { y | n }] [-l 'p1 p2 xinc yinc'] [-p filename] [-rc filename] [-rd filename] [-s subtitle] [-t title] [-v]"
#define VERSION "1.0"			/*Version number*/

/*Index for image and pattern variables*/
#define IMAGE           0               /*Object is for image*/
#define PATTERN         1               /*Object is for pattern*/

/*Pattern matching algorithms*/
#define SPACE_DOMAIN	0		/*Correlation (space domain)*/
#define FREQ_DOMAIN	1		/*Correlation (frequency domain)*/
#define LPS		2		/*Linear pixel shuffling*/
#define RANDOM		3		/*Random*/

#define NUM_ALGS	4		/*Number of algorithms*/

/*Inversion states*/
#define AUTO_INVERT	0		/*Automatically determine inversion*/
#define NO_INVERT	1		/*Don't invert*/
#define YES_INVERT	2		/*Invert*/

/*Flag bits for batch mode*/
#define BATCHMODE	0x01		/*Running in batch mode*/
#define BATCHFILE	0x02		/*Running from a file, not interactive*/

/*Timing structure*/
typedef struct _time {
	long hours;			/*Number of hours*/
	long minutes;			/*Number of minutes*/
	long seconds;			/*Number of seconds*/
} TIME;

/*Pattern resulting info structure*/
typedef struct _info {
	int alg;			/*Algorithm that ran*/
	long x;				/*X coordinate*/
	long y;				/*Y coordinate*/
	long p1;			/*1st maximum number*/
	long p2;			/*2nd maximum number*/
	int k1;				/*X increment for LPS*/
	int k2;				/*Y increment for LPS*/
	long seed;			/*Random seed generator*/
	TIME time;			/*Run time*/
} INFO;

extern int errno;			/*Number of last error*/
extern int batchflag;			/*Batch mode flags*/
extern int writetimer;			/*Timer pipe file descriptor*/
extern int child;			/*Child timer process*/
extern invert_bias[2];			/*Bias auto invert*/
extern char *program;			/*Name of executing program*/
extern char *dist_results;		/*Distance results filename*/
extern char *filename[2];		/*Image, pattern filenames*/
extern char *title;			/*Graph title*/
extern char *subtitle;			/*Graph subtitle*/
extern unsigned char *data[2];		/*Unmapped data*/
extern HEADER header[];			/*Internal image header*/
extern PATTERN_INFO pattern_info;	/*Pattern information*/
extern INFO info;			/*Matched pattern resulting info*/

extern double corr();			/*Correlation routine*/

/*Macro for converting [x,y] to index in data array*/
#define cvt(x,y,whichheader) ((y * header[whichheader].pixels + x))

/*Dealing with the batch mode flag*/
#define setbit(bit) (batchflag |= bit)
#define clearbit(bit) (batchflag &= ~bit)
#define isbitset(bit) (batchflag & bit)
#define isbitclear(bit) (~batchflag & bit)

/*Batch mode prompt*/
#define prompt() (printf("%s> ",program))

extern void display_results();

