.ll 6.5i
.TH SHERLOCK N "02-May-94"
.SH NAME
sherlock - compare pattern recognition algorithms
.SH SYNOPSIS
sherlock -i filename [-b { batchfile | - }] [-ii { y | n }] [-ip { y | n }] [-l 'p1 p2 xinc yinc'] [-p filename] [-rc filename] [-rd filename] [-rs number] [-s subtitle] [-t title] [-v]
.SH DESCRIPTION
\f2Sherlock\f1 is an Open Look application that can compare (currently) four
different pattern recognition algorithms.  The algorithms are:  Space Domain
Correlation, Frequency Domain Correlation, Linear Pixel Shuffling, and Random
Ordering.  Each is described in more detail below.

\f2Sherlock\f1 can run either in graphical UI mode or batch mode.  Batch mode
can process commands from a file or interactively from the keyboard.  Both
modes are detailed below in their own sections.

Images are limited to 8 bit/pixel grayscale (0 = black, 255 = white).

Two result files are created for each algorithm run, the correlation results,
and the distance results.  The correlation values are the correlation values
between image and pattern at each pixel location within the image.  These are
not TRUE correlation values in the sense that in order to speed up processing,
it was faster to find the difference (rather than the correlation) between image
and pattern and then invert so that the maximum value showed the smallest
difference.  So, in effect, these values are really differences rather than
correlations.  The other results file is comprised of the Euclidean distance
between where \f2Sherlock\f1 thinks the pattern is and where it really is.
Distance is mathematically defined as the distance between two points given by:

.ce
.EQ
sqrt  {(x sub 1 -x sub {0}) sup 2 +(y sub 1 -y sub {0}) sup 2}
.EN

where:

.ta 2
.in +5
.EQ
x sub 0
.EN
	- true X coordinate of pattern
.br
.EQ
x sub 1
.EN
	- estimated X coordinate by \f2Sherlock\f1
.br
.EQ
y sub 0
.EN
	- true Y coordinate of pattern
.br
.EQ
y sub 1
.EN
	- estimated Y coordinate by \f2Sherlock\f1
.in -5

When an algorithm starts, the estimated position of the pattern is set to (0,0).
This means that if the pattern is near location (0,0), the distance will
start off small as opposed to a pattern that is further away.

The results files are formatted to be read in with \f3ACE/gr\f1, a free 2D
graphing program for exploratory data analysis (xvgr - Open Look version or
xmgr - Motif version) written by Paul Turner (of the Oregon Graduate Institute,
email pturner@ese.ogi.edu).  This enables the results to be graphed quickly.

After an algorithm has run, information about it is presented.  In batch mode,
it is written to the screen.  In UI mode, a window pops up with the information.
The information given is the same in either case.  This includes:

.ta 10
.in +5
.nf
Test	- algorithm than ran
Ratio	- ratio between image and pattern (image:pattern)
Time	- time spent in running the algorithm (h:m:s)
Pos	- position of where a match was made
Image	- filename of the image used
Pattern	- filename of the pattern used (or "from image" if cropped)
.fi
.in -5

In the case of Linear Pixel Shuffling, the parameters (Parms) and the
increments (Kn) used are given.  In the case of Random Ordering, the seed
(Seed) for the random number generator is given.
.SH OPTIONS
.IP "-i filename"
is the name of the \f3image file\f1 to use. This option is required.  This
file must be in \f2Sherlock\f1 internal format (see the header_util utility).

.IP "-b { batchfile | - }"
specifies to run in \f3batchmode\f1.  No graphical UI is shown in this mode.
A file of commands (\f3batchfile\f1) can be given to be interpreted.  If a "-"
is given for a filename, batch mode commands are read in from \f3stdin\f1.  See
the BATCH MODE section below.  A graphical UI will be presented if this option
isn't given.
.IP "-ii { y | n }"
Determine how to invert the \f3image\f1 for the Frequency Domain Correlation
algorithm.  See the section on this algorithm below for more information on
the inversion.  A "y" will always invert and a "n" will never invert.  By
default, \f2Sherlock\f1 will automatically determine if inversion is needed.
.IP "-ip { y | n }"
Determine how to invert the \f3pattern\f1 for the Frequency Domain Correlation
algorithm.  See the section on this algorithm below for more information on
the inversion.  A "y" will always invert and a "n" will never invert.  By
default, \f2Sherlock\f1 will automatically determine if inversion is needed.
.IP "-l 'p1 p2 xinc ycin'
are the \f3parameters\f1 to use for the Linear Pixel Shuffling algorithm.  "p1"
is the first maximum number.  "p2" is the second maximum number.  "xinc" is the
X increment.  "yinc" is the Y increment.  All numbers are integers.
See the LINEAR PIXEL SHUFFLING section below for more details on what these
numbers should be and why they are needed.  If not given, they are calculated
using a modified Fibonacci sequence.
.IP "-p filename"
is the name of the \f3pattern file\f1 to use.  This file must be in
\f2Sherlock\f1 internal format (see the header_util utility).
.IP "-rc filename"
is the name to save the \f3correlation data\f1 to.  This file is written out
to a format to be read in by \f3ACE/gr\f1.  Default is "corr_results.data".
.IP "-rd filename"
is the name to save the \f3distance data\f1 to.  This file is written out to a
format to be read in by \f3ACE/gr\f1.  Default is "dist_results.data".
.IP "-rs number"
is the \f3random seed\f1 to use in the Random Ordering algorithm.  Default is
to use the number of seconds since 00:00:00 GMT, Jan. 1, 1970 as returned
by the time(3) function.
.IP "-s subtitle"
is the \f3subtitle\f1 for the output graphs (corr_results.data and
dist_results.data).  Default is to have no subtitle on the graphs.
.IP "-t title"
is the \f3title\f1 for the output graphs (corr_results.data and
dist_results.data).  Default is to have no title on the graphs.
.IP "-v"
displays the \f3version\f1 number and quits.  No further processing is done.
.SH GRAPHICAL UI MODE
By default, \f2Sherlock\f1 will start in the interactive \f3graphical UI
mode\f1.  The image will be displayed with a command panel (of button menus)
above it.  If the pattern is specified on the command line, it is displayed
in it's own window.  The left footer of the image and pattern windows
displays \f2Sherlock's\f1 current state (i.e. Ready, Loading..., etc.).

The button menus on the command panel are labeled \f3File\f1, \f3Pattern\f1,
and \f3Run\f1.

The \f3File\f1 button menu contains I/O options and "Exit".  Within this
menu, the following choices are available:

.ta 20
.in +5
.nf
Load Image...	- load a new image into memory
Load Pattern...	- load a (new) pattern into memory
Save Pattern...	- save a pattern to disk
Exit	- exit \f2Sherlock\f1
.fi
.in -5

Selecting the load/save options will bring up a window asking for a directory
and a filename.  After this information is entered, the user can either click
the "Apply" button to apply the input or "Cancel" button to discard the changes.
Values entered for directory and filename will remain with the individual
windows (i.e. a filename given to the Load Image window will appear the next
time this window is displayed and won't appear for the other I/O windows).

The "Save Pattern..." choice is grayed out until there is a pattern in
memory to save.

Files to be loaded must be in \f2Sherlock\f1 internal format (see  the
header_util utility).  Files are saved in \f2Sherlock\f1 internal format.

The \f3Pattern\f1 button menu contains choices for dealing with patterns.
The "Select" choice will crop the pattern from the image.  The mouse can be
used to create a crop box (see Mouse Operations paragraph below).  The "Display"
choice will display a pattern that's in memory.  Both choices are grayed out
until needed (for "Select" that's creating a crop box, for "Display" that's
having a pattern in memory to display).

The \f3Run\f1 button contains the algorithms to run.  The four choices
are:  "Space Domain Correlation", "Frequency Domain Correlation", "Linear
Pixel Shuffling", and "Random Ordering".  This button is grayed out until
there is a pattern in memory so that the algorithms can run.

A gauge is displayed when an algorithm is being executed.  This gauge displays
how much of the algorithm run time has been spent and how much more there is
to go.

\f3Mouse Operations\f1 are as follows.  The crop box used to select a pattern
from the image is drawn by holding down the left button.  The box can be
stretched by moving the mouse with the left button held down.  As soon as the
left button is released, the crop box can no longer be adjusted.  If the
left button isn't being held down, the current mouse position is displayed in
the right footer of the image window.  This allows the user to see how good
the algorithm worked to find the center location of the pattern.

The colormap is set to a grayscale colormap (and so the images are limited
to 8 bit/pixel grayscale).  Closest colors are used if a certain grayscale
is unavailable (meaning that a white background can look slightly grayish).
The data used in the calculations or saved to disk ARE the actual
values, not the colormapped values.
.SH BATCH MODE
\f2Sherlock\f1 can also run non-interactively in \f3batch mode\f1.  In this
mode, no images are displayed.  This mode can be used in a pure ASCII
environment.

A file of commands can be given to be interpreted.  If a "-" (dash) is
given as the filename for the "-b" option, commands are read from \f3stdin\f1.

The following commands are available:

.IP "crop x y width height"
Crop out an area from the image to become the pattern.  \f3X\f1 and \f3Y\f1
are the (X,Y) starting coordinates of the area to crop.  \f3Width\f1 is the
width (in pixels).  \f3Height\f1 is the height (in pixels).  For example,
the command:

.in +5
crop 10 20 100 150
.in -5

will crop an area from the image starting at pixel position (10,20) and crop
out an area 100 pixels width by 150 pixels height.  This cropped area becomes
the pattern.
.IP "exit"
Exit the program.
.IP "invert type {yes|no|auto}"
Determine how to invert the image or pattern.  This is the same as the "-ii" and "-ip"
switches above (with the addition of "auto").  The \f3type\f1 parameter determines which
data is affected.  It can be "image" or "pattern".  Setting to "yes" will always invert.
Setting to "no" will never invert.  Setting to "auto" will let \f2Sherlock\f1 try to
automatically determine if inversion is needed.  "Auto" is the default setting.
An example:

.in +5
invert pattern no
.in -5

will cause \f2Sherlock\f1 to not invert the pattern.  This is useful if the
pattern's (or image's) first pixel isn't the background color.  See the
FREQUENCY DOMAIN section below for an explanation on why inversion
is sometimes necessary.
.IP "load type filename"
Load a file into memory.  The \f3type\f1 parameter determines what is being
read in (image or pattern).  \f3Type\f1 can be "image" or "pattern".  The
\f3filename\f1 is the name of the file to read in.  For example, the
command:

.in +5
load image abc.raw
.in -5

will load the image in "abc.raw" into memory.
.IP "run algorithm"
Run the selected algorithm.  The possible choices for \f3algorithm\f1 are:

.ta 5 25
.nf
	space	- space domain correlation
	frequency (or freq)	- frequency domain correlation
	lps	- linear pixel shuffling
	random	- random ordering
.fi

For example, the command:

.in +5
run lps
.in -5

will run the Linear Pixel Shuffling algorithm.
.IP "save type filename"
Save a file to disk.  The \f3type\f1 parameter determines what is being
written out (image or pattern).  \f3Type\f1 can be "image" or "pattern".
This version currently only supports saving patterns, so in effect \f3type\f1
is useless at this point (i.e. the pattern will be saved regardless of what
\f3type\f1 is set to).  The \f3filename\f1 is the name of the file to read
in.  For example, the command:

.in +5
save pattern abc.raw
.in -5

will save the pattern from memory to "abc.raw".
.IP "seed = { number | ? }"
Set the random seed to the number indicated.  If the '?' is given instead
of a number, \f2Sherlock\f1 will get the random seed for the Random
Ordering algorithm from the time(3) function.  For example, the command:

.in +5
seed = 471
.in -5

will set the seed for the random number generator to 471.
.IP "sleep seconds"
Sleep for the time indicated.  The time argument (\f3seconds\f1) as you
might have guessed are in seconds.  For example, the command:

.in +5
sleep 120
.in -5

will sleep for 120 seconds (2 minutes).
.IP "!command"
Execute a shell escape.  \f3Command\f1 is the command to run.  For example, the
command:

.in +5
!ls -l
.in -5

will run the "ls -l" command in the current directory.
.SH SPACE DOMAIN
The equation for space domain correlation is:

.ce
.EQ
g ( q vec ) ~=~ int from D f[ p vec ] h[ p vec - q vec ] d {p vec}
.EN

What this equation means is that for every pixel location (
.EQ
p vec
.EN
) in the image (D), find the cross-correlation to the pattern at that location
by summing up the pixel-by-pixel multiplications.  This is done starting
at location [0,0] and moving one pixel at a time across each scanline (i.e.
[0,0], then [0,1]...[0,N], then [2,0]...[2,N] and so on til [N,N]).

Instead of the normal correlation of adding up the pixel-by-pixel
multiplies, \f2Sherlock\f1 sums the absolute value of the
difference between corresponding pixels.  This is given by:

.ce
.EQ
g ( q vec ) ~=~ sum from "pattern" left | f[ p vec ]
- h[ p vec - q vec ] right |
.EN

\f2Sherlock\f1 then looks for a minimum to find the greatest correlation (i.e.
the least difference between matrices).  This speeds up the processing by not
having to:

.in +5
1) calculate the average for pattern and image
.br
2) subtracting that average  from each pixel
.in -5

The average needed to be subtracted because good (and false) correlations
results from coming from the totally white areas within the image.

Also, by not having to subtract the average, \f2Sherlock\f1 can do the
subtraction with integers, not floating point numbers.

The steps for this algorithm are as follows:

.in +5
1) starting at [row=0,col=0], find the difference
.br
2) check difference value for minimum
.br
3) move on to the next pixel position ([row,col+1] or if at the end of
a scanline, [row+1,0])
.in -5
.SH FREQUENCY DOMAIN
Instead of the correlation being computed in the space domain, it is
computed in the frequency domain.  This is done by taking the Fourier
Transform of image and pattern, multiplying the transforms together,
and then taking the inverse Fourier Transform.  In mathematical
theory, this gives you the same results as Space Domain Correlation.
This being reality, things are done a little differently.

The Fourier Transform function used assumes that the origin is in the
middle of the scanline.  However, \f2Sherlock\f1 calculates using the
origin as the first pixel in the scanline.  In order to match up
perfectly with the Space Domain Correlation, either "beercanning" or
"checkerboarding" would have to be done so that the Fourier Transform
function sees the data with the origin in the middle.  In order to
save processing time, \f2Sherlock\f1 doesn't do either.  It
compensates by calculating the origin (middle) location of the pattern
within the image by knowing that the Fourier Transform function will
always return the maximum correlation location at the upper left
corner of the pattern.  Had \f2Sherlock\f1 done either "beercanning"
or "checkerboarding", the maximum correlation would have been found in
the center of the pattern, as it does for Space Domain.

In order for the Frequency Domain Correlation to work, \f2Sherlock\f1
needs a black (pixel value 0) background with a nonblack object.  This
is so zero padding of black can be applied to the pattern and image to boost the
size to that of a power of 2 (so that the Fast Fourier Transform
function can work as fast as possible).  Therefore, the first pixel
(location [0,0]) is checked to see if it's equal to 0.  If it is,
\f2Sherlock\f1 assumes the background is black and the object is nonblack.  If
it isn't, \f2Sherlock\f1 inverts the image (or pattern).  This assumes that
the pattern won't be covering position (0,0).  If it is, or some reason, the
auto detection isn't working properly, the inversion behavior can be
controlled by switches (-ii and -ip) and in the case of batch mode, by
commands (invert type {yes|no|auto}).

The steps for this algorithm are as follows:

.in +5
1) find power of 2 size equal to or greater than the largest dimension in
pattern and image
.br
2) zero pad pattern and image to that power of 2 size
.br
3) calculate Fourier Transform of image and pattern
.br
4) multiply transforms on a pixel-by-pixel basis
.br
5) take inverse Fourier Transform of multiplication
.br
6) search for maximum in inversed transform
.in -5
.SH LINEAR PIXEL SHUFFLING
Linear Pixel Shuffling (LPS) is a method to get a semi-random ordering of
pixels.  Unlike Space Domain described above which has a sequential
pixel order, LPS jumps around without (hopefully) hitting the same
pixel location twice.  It wraps around the image by use of modular
arithmetic.  LPS tries to space the jumps evenly (instead of a real
random ordering).  A simple arithmetic progression is needed to
determine the next pixel location.  Refer to "Linear Pixel Shuffling
for Image Proccessing, an Introduction" and "Linear Pixel Shuffling
Made Easy" by Peter Anderson (pga@cs.rit.edu) for more information.

The change in position is calculated as:

.in +5
New_X_coord = (Old_X_coord + X_increment) % x_max;
.br
New_Y_coord = (Old_Y_coord + Y_increment) % y_max;
.in -5

Both the increments and the maximum values can be given on the command
line (-l option).  The maximum values (x_max, y_max) are equal to or greater
than the image dimensions.

The algorithm is the same as for SPACE DOMAIN (see above) except for the
method of selecting the next pixel within the image (step 3).  Instead, the
change in position is calculated by using the equation listed above.
.SH RANDOM ORDERING
Random ordering is like SPACE DOMAIN (described above), except
that selecting the next pixel within the image (step 3) is done by randomly
choosing the next pixel.
.SH SEE ALSO
xmgr(1), xvgr(1), header_util(n)

Fundamentals of Continuous Linear Systems
.br
Mathematics of Discrete Linear Systems
.br
(Roger L. Easton, Jr., rlepci@ritvax.isc.rit.edu)

Linear Pixel Shuffling for Image Proccessing, an Introduction
.br
Linear Pixel Shuffling Made Easy
.br
(Peter Anderson, pga@cs.rit.edu)
.SH BUGS
Images are limited to 8 bit/pixel grayscale.

The allocation of the colormap is pitiful.

No scrollbars on the canvases (displayed images limited to dimensions of
the screen).
.SH AUTHOR
Adam Stein (adam@iset.scan.mc.xerox.com)
