#include <values.h>
#include "sherlock.h"

/*Calculate a space domain correlation.  The steps are as follows:

  1) starting at [0,0], find the correlation at each pixel position
  2) check correlation value for minimum
  3) move on to next position and repeat til all pixels have been
     covered

  Because of the way the correlation is calculated, (it's actually a
  difference rather than a correlation) we look for the smallest
  value, rather than the largest.

  Inputs:  none
  Outputs: none
  Locals:  loopx     - loop through pixels
	   loopy     - loop through scanlines
	   min       - minimum correlation value
	   position  - position in correlation matrix to store value
	   res       - result of correlation
  Globals: header    - image/pattern header information arrays
	   BATCHMODE - flag bit indicating Sherlock is running in batch mode
	   IMAGE     - reference the image element of an array
	   MAXDOUBLE - maximum value of a double variable
*/
space()
{
	register long loopy,loopx,position;
	register double min,res;

	/*Set initial conditions*/
	min = MAXDOUBLE;
	position = 0;

	/*Loop through pixels*/
	for(loopy = 0;loopy < header[IMAGE].scanlines;++loopy) {
	  if(isbitclear(BATCHMODE))
	    update_gauge((double) loopy / (double) header[IMAGE].scanlines);

	  /*Calculate correlation and compare for minimum*/
	  for(loopx = 0;loopx < header[IMAGE].pixels;++loopx,++position)
	    if((res = corr(loopx,loopy,position)) < min) {
	      min = res;
	      info.x = loopx;
	      info.y = loopy;
	    }
	}
}

